/*
 * copy.c
 * : open, read, write, close 함수를 이용해서 파일을 복사하는 코드.
 *  Created on: 2018. 5. 10.
 *      Author: minseo
 */

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#define BUFSIZE 1024

void FileCopy(char* source, char* target);

int main(int argc, char** argv) {

	if (argc != 3) {
	    printf("Usage: copy source target \n ");
	    return 1;
	}

	FileCopy(argv[1], argv[2]);

	return 0;
}

// source 파일을 읽어 target 파일에 복사
void FileCopy(char* source, char* target) {
    int source_file_fd = 0;
    int target_file_fd = 0;
    int source_bytes = 0;
    int target_bytes = 0;
    int total_source_bytes = 0;
    int total_target_bytes = 0;
    char data_buf[BUFSIZE] = { '\0', };

    // source file 을 읽기 모드로 열어서 읽는다.
    if ((source_file_fd = open(source, O_RDONLY)) == -1) {
        perror("source file open error");
        exit(1);
    }

    // target file 을  쓰기모드로 연다. 만약 "recv" 파일이 없다면 만들어서 쓰기모드로 연다.
    if ((target_file_fd = open(target, O_WRONLY | O_CREAT, 0644)) == -1) {
        perror("target file open error");
        exit(1);
    }

    while (1) {
        // 1024 BYTE 만큼 source 파일에서 읽어 data_buf에 저장한다.
        if ((source_bytes = read(source_file_fd, data_buf, BUFSIZE)) == -1) {
            perror("source file read error");
            exit(1);
        }

        // 파일에서 읽은 data가 존재하면 파일에 쓰고, 그렇지 않으면 파일 복사가 잘 되었는지 확인한다.
        if (source_bytes != 0) {
            total_source_bytes += source_bytes;

            // 1024 BYTE 만큼 target 파일에 data_buf를 쓴다.
            if ((target_bytes = write(target_file_fd, data_buf, source_bytes)) == -1) {
                perror("taget file write error");
                exit(1);
            }
            total_target_bytes += target_bytes;

        } else {
            break;
        }
    }

    // 기존 데이터와 복사된 데이터의 크기가 맞지 않으면 파일 오류를 출력하고, 복사한 파일을 삭제한다.
    if(total_source_bytes != total_target_bytes) {
        printf("file copy error.");
        remove(target);
    }

    close(source_file_fd);
    close(target_file_fd);
}


