/*
 * copy.c
 * : fopen, fread, fwrite, fclose 함수를 이용해서 파일을 복사하는 코드.
 *  Created on: 2018. 5. 10.
 *      Author: minseo
 */

#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#define BUFSIZE 1024

void FileCopy(char* source, char* target);

int main(int argc, char** argv) {

    if (argc != 3) {
        printf("Usage: copy source target \n ");
        return 1;
    }

	FileCopy(argv[1], argv[2]);

	return 0;
}

// source 파일을 읽어 target 파일에 복사
void FileCopy(char* source, char* target) {
	FILE* source_fp;
	FILE* target_fp;
	int source_bytes;
	int target_bytes;
	int total_source_bytes = 0;
	int total_target_bytes = 0;
	char data_buf[BUFSIZE];

	// source 파일을 읽기모드로 연다.
	if ((source_fp = fopen(source, "r")) == NULL) {
		perror("source file open error");
		exit(1);
	}

	// target 파일을 쓰기모드로 연다. open과 다르게 target 파일이 없으면 알아서 생성해서 쓰기모드로 연다.
	if ((target_fp = fopen(target, "w")) == NULL) {
		perror("target file open error");
		exit(1);
	}

	while (1) {
		// source 파일의 1024 BYTE를 char단위로 읽어 data_buf에 저장한다.
		if((source_bytes = fread(data_buf, sizeof(char), BUFSIZE, source_fp)) == -1) {
		    perror("source file read error");
		    exit(1);
		}

		total_source_bytes += source_bytes;

		// source 파일에서 더이상 읽을 데이터가 없으면 파일 복사가 잘 되었는지 확인한다.
		if (source_bytes == 0) {
			break;
		}

		// data_buf 저장된 데이터를 source_bytes만큼 char단위로 읽어 target 파일에 쓴다.
		if((target_bytes = fwrite(data_buf, sizeof(char), source_bytes, target_fp)) < source_bytes) {
			perror("target file write error");
			exit(1);
		}
		total_target_bytes += target_bytes;
	}

	// 기존 데이터와 복사된 데이터의 크기가 맞지 않으면 파일 오류를 출력하고, 복사한 파일을 삭제한다.
	if (total_source_bytes != total_target_bytes) {
		printf("file copy error.");
		remove(target);
	}

	fclose(source_fp);
	fclose(target_fp);
}
